/***************************************************************************
                          k3bfileview.cpp  -  description
                             -------------------
    begin                : Sun Oct 28 2001
    copyright            : (C) 2001 by Sebastian Trueg
    email                : trueg@informatik.uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "k3bfileview.h"
#include "k3b.h"
#include "k3baudioplayer.h"
#include "k3bdoc.h"
#include "k3bdiroperator.h"
#include "k3btoolbox.h"

#include <qwidget.h>
#include <qdragobject.h>
#include <qlayout.h>
#include <qdir.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qtoolbutton.h>

#include <kfiledetailview.h>
#include <klistview.h>
#include <kaction.h>
#include <ktoolbar.h>
#include <ktoolbarbutton.h>
#include <kurl.h>
#include <kurldrag.h>
#include <kfilefiltercombo.h>
#include <klocale.h>
#include <kfileitem.h>
#include <kmessagebox.h>




K3bFileView::K3bFileView(QWidget *parent, const char *name ) 
  : K3bCdContentsView(parent,name) 
{
  setupGUI();
}


K3bFileView::~K3bFileView()
{
}


KActionCollection* K3bFileView::actionCollection() const
{
  return m_dirOp->actionCollection();
}


void K3bFileView::setupGUI()
{
  QVBoxLayout* layout = new QVBoxLayout( this );
  //  layout->setAutoAdd( true );

  m_dirOp = new K3bDirOperator( QDir::home().absPath(), this );
  m_dirOp->readConfig( k3bMain()->config(), "file view" );
  m_dirOp->setMode( KFile::Files );
  m_dirOp->setView( KFile::Default );

  K3bToolBox* toolBox = new K3bToolBox( this );

  layout->addWidget( m_dirOp );
  layout->addWidget( toolBox );
  layout->setStretchFactor( m_dirOp, 1 );


  // setup actions
  KAction* actionPlay = new KAction( i18n("Play"), "1rightarrow", 0, this, SLOT(slotAudioFilePlay()), 
				     m_dirOp->actionCollection(), "audio_file_play");
  KAction* actionEnqueue = new KAction( i18n("En&queue"), "1rightarrow", 0, this, SLOT(slotAudioFileEnqueue()), 
					m_dirOp->actionCollection(), "audio_file_enqueue");
  KAction* actionAddFilesToProject = new KAction( i18n("&Add to Project"), SHIFT+Key_Return, 
						  this, SLOT(slotAddFilesToProject()), 
						  m_dirOp->actionCollection(), "add_file_to_project");

  KAction* actionHome = m_dirOp->actionCollection()->action("home");
  KAction* actionBack = m_dirOp->actionCollection()->action("back");
  KAction* actionUp = m_dirOp->actionCollection()->action("up");
  KAction* actionReload = m_dirOp->actionCollection()->action("reload");


  toolBox->addButton( actionUp );
  toolBox->addButton( actionBack );
  toolBox->addButton( actionHome );
  toolBox->addButton( actionReload );
  toolBox->addSpacing();
  toolBox->addButton( actionPlay );
  toolBox->addSpacing();


  // insert actions into diroperator menu
  KActionMenu* dirOpMenu = (KActionMenu*)m_dirOp->actionCollection()->action("popupMenu");
  dirOpMenu->insert( actionAddFilesToProject, 0 );
  dirOpMenu->insert( new KActionSeparator( m_dirOp->actionCollection() ), 1 );
  dirOpMenu->insert( actionPlay, 2 );
  dirOpMenu->insert( actionEnqueue, 3 );
  dirOpMenu->insert( new KActionSeparator( m_dirOp->actionCollection() ), 4 );

  // check if some actions should be enabled
  connect( dirOpMenu, SIGNAL(activated()), this, SLOT(slotCheckActions()) );

  // create filter selection combobox
  toolBox->addLabel( i18n("Filter:") );
  m_filterWidget = new KFileFilterCombo( toolBox, "filterwidget" );
  toolBox->addWidget( m_filterWidget );

  m_filterWidget->setEditable( true );
  QString filter = i18n("*|All files");
  filter += "\n" + i18n("audio/x-mp3 application/x-ogg audio/wav |Sound files");
  filter += "\n" + i18n("audio/x-mp3 |MP3 sound files");
  filter += "\n" + i18n("video/mpeg |MPEG video files");
  filter += "\n" + i18n("application/x-ogg |Ogg Vorbis sound files");
  m_filterWidget->setFilter(filter);

  connect( m_filterWidget, SIGNAL(filterChanged()), SLOT(slotFilterChanged()) );

  connect( m_dirOp, SIGNAL(fileHighlighted(const KFileItem*)), this, SLOT(slotFileHighlighted(const KFileItem*)) );
  connect( m_dirOp, SIGNAL(urlEntered(const KURL&)), this, SIGNAL(urlEntered(const KURL&)) );
  connect( m_dirOp, SIGNAL(doubleClicked(KFileItem*)), this, SLOT(slotAddFilesToProject()) );

  actionPlay->setEnabled( false );
}


void K3bFileView::setUrl(const KURL& url, bool forward)
{
  m_dirOp->setURL( url, forward );
}


void K3bFileView::slotFileHighlighted( const KFileItem* )
{
  // check if there are audio files under the selected ones
  bool play = false;
  for( QListIterator<KFileItem> it( *(m_dirOp->selectedItems()) ); it.current(); ++it ) {
    if( k3bMain()->audioPlayer()->supportsMimetype(it.current()->mimetype()) ) {
      play = true;
      break;
    }
  }

  if( play ) {
    m_dirOp->actionCollection()->action( "audio_file_play" )->setEnabled( true );
    m_dirOp->actionCollection()->action( "audio_file_enqueue" )->setEnabled( true );
  }
  else {
    m_dirOp->actionCollection()->action( "audio_file_play" )->setEnabled( false );
    m_dirOp->actionCollection()->action( "audio_file_enqueue" )->setEnabled( false );
  }
}


void K3bFileView::slotAudioFilePlay()
{
  // play selected audio files
  QStringList files;

  for( QListIterator<KFileItem> it( *(m_dirOp->selectedItems()) ); it.current(); ++it ) {
    if( k3bMain()->audioPlayer()->supportsMimetype(it.current()->mimetype()) ) {
      files.append( it.current()->url().path() );
    }
  }

  if( !files.isEmpty() ) {
    if( !k3bMain()->audioPlayer()->isVisible() )
      k3bMain()->slotViewAudioPlayer();
    k3bMain()->audioPlayer()->playFiles( files );
  }
}


void K3bFileView::slotAudioFileEnqueue()
{
  // play selected audio files
  QStringList files;

  for( QListIterator<KFileItem> it( *(m_dirOp->selectedItems()) ); it.current(); ++it ) {
    if( k3bMain()->audioPlayer()->supportsMimetype(it.current()->mimetype()) ) {
      files.append( it.current()->url().path() );
    }
  }

  if( !files.isEmpty() )
    k3bMain()->audioPlayer()->enqueueFiles( files );
}


void K3bFileView::slotAddFilesToProject()
{
  if( !k3bMain()->activeDoc() ) {
    //KMessageBox::error( this, i18n("Please create a project before adding files"), i18n("No Active Project"));
    int project = KMessageBox::questionYesNoCancel( this, i18n("There is no active project. Do you want to create one?"),
       i18n("No Active Project"), i18n("Audio Project"), i18n("Data Project") );
    switch(project) {
    case 3:
      k3bMain()->slotNewAudioDoc();
      break;
    case 4:
      k3bMain()->slotNewDataDoc();
      break;
    default:
      break;
    }
  }
  if( k3bMain()->activeDoc() ){
    KURL::List files;
    for( QListIterator<KFileItem> it( *(m_dirOp->selectedItems()) ); it.current(); ++it ) {
      files.append( it.current()->url() );
    }    
    if( !files.isEmpty() )
      k3bMain()->activeDoc()->addUrls( files );
  } else {
      kdDebug() << "(K3bFileView) Adding files to project canceled." << endl;
  }
}


void K3bFileView::slotFilterChanged()
{
  QString filter = m_filterWidget->currentFilter();
  m_dirOp->clearFilter();

  if( filter.find( '/' ) > -1 ) {
    QStringList types = QStringList::split( " ", filter );
    types.prepend( "inode/directory" );
    m_dirOp->setMimeFilter( types );
  }
  else
    m_dirOp->setNameFilter( filter );
  
  m_dirOp->rereadDir();
  //  emit filterChanged( filter );
}


void K3bFileView::slotCheckActions()
{
  m_dirOp->actionCollection()->action("add_file_to_project")->setEnabled( k3bMain()->activeView() != 0 );
}


void K3bFileView::reload()
{
  m_dirOp->actionCollection()->action("reload")->activate();
}


void K3bFileView::saveConfig( KConfig* c )
{
  m_dirOp->writeConfig( c, "file view" );
}


#include "k3bfileview.moc"
