/***************************************************************************
                          k3bview.h  -  description
                             -------------------
    begin                : Mon Mar 26 15:30:59 CEST 2001
    copyright            : (C) 2001 by Sebastian Trueg
    email                : trueg@informatik.uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef K3BVIEW_H
#define K3BVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// include files for Qt
#include <qwidget.h>

class K3bDoc;
class K3bProjectBurnDialog;
class KActionCollection;


/** The K3bView class provides the view widget for the document instance connected to it and is displayed
 * as a MDI child window in the main view area of the K3bMainWindow class instance. The K3bMainWindow class also has an eventFilter()
 * method that gets installed on every K3bView instance to control events of the type QEvent::Close.	
 * The document connected to the view instance keeps a list of all view that represent the document contents as there
 * can be more than one view. Views get created in K3bMainWindow::createClient() and automatically added to the list of views.
 * The K3bView class inherits QWidget as a base. Another possible inheritance besides specialized widgets could be
 * QMainWindow so that you can easily set up the main area of your view by setting another view
 * as main widget (QMainWindow::setMainWidget() ).
 * NOTE: The close event always has to be empty (DON`T CALL QWidget::closeEvent(e) in closeEvent()) because the installed
 * event filter can only manage a forward implementation. If the QCloseEvent is received by the K3bView, the overwritten
 * event handler has to do nothing as the eventFilter has set accept() or ignore() already. If QWidget::closeEvent() is called
 * again, the default event handler will accept the close event and the window gets destroyed even if the installed eventFilter
 * has set the event to be ignored.	
 * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
 * @version KDevelop version 1.3 code generation
 */
class K3bView : public QWidget
{
  Q_OBJECT

    friend class K3bDoc;

 public:
  /** Constructor for the view
   * @param pDoc  your document instance that the view represents. Create a document before calling the constructor
   * or connect an already existing document to a new MDI child widget.*/
  K3bView(K3bDoc* pDoc, QWidget* parent, const char *name = 0 );

  /** Destructor for the main view */
  ~K3bView();
	
  /** returns a pointer to the document connected to the view*/
  K3bDoc *getDocument() const;
	
  virtual void burnDialog( bool withWritingButton = true ) = 0;

  virtual KActionCollection* actionCollection() const;

 protected:
  /** overwritten QWidget::closeEvent() to catch closing views. Does nothing, as the closeEvents for
   * K3bView's are processed by K3bMainWindow::eventFilter(), so this overwitten closeEvent is necessary
   * and has to be empty. Don't overwrite this method !
   */
  virtual void closeEvent(QCloseEvent* e);
	
  /** The document connected to the view, specified in the constructor */
  K3bDoc *doc;
  KActionCollection* m_actionCollection;
};

#endif // K3BVIEW_H
